#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
import uuid

import appier

HELLO_WORLD_B64 = "SGVsbG8gV29ybGQAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAQAAAAEAAABAAQAAAA\
AAAAAAAABDYWxpYnJpAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACQAAAAMAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA\
AAAAAAAAAwAAABIZWxsbyBXb3JsZAA="

class NodeController(appier.Controller):

    @appier.route("/nodes", "GET", json = True)
    @appier.ensure(token = "admin")
    def list(self):
        return self.owner.nodes

    @appier.route("/nodes/<str:id>", "POST", json = True)
    @appier.ensure(token = "admin")
    def create(self, id):
        node = appier.get_object()
        self.owner.nodes[id] = node

    @appier.route("/nodes/<str:id>", "GET", json = True)
    @appier.ensure(token = "admin")
    def show(self, id):
        return self.owner.nodes[id]

    @appier.route("/nodes/<str:id>/jobs", "GET", json = True)
    @appier.ensure(token = "admin")
    def jobs(self, id):
        self.request.set_content_type("application/json")
        for value in appier.header_a(): yield value
        for value in self.wait_jobs(id): yield value

    @appier.route("/nodes/<str:id>/jobs_peek", "GET", json = True)
    @appier.ensure(token = "admin")
    def jobs_peek(self, id):
        jobs = self.owner.jobs.get(id, [])
        return jobs

    @appier.route("/nodes/<str:id>/print", ("GET", "POST"), json = True)
    @appier.ensure(token = "admin")
    def print_default(self, id):
        data_b64 = self.field("data_b64", mandatory = True, not_empty = True)
        name = self.field("name", None)
        name = name or str(uuid.uuid4())
        job = dict(data_b64 = data_b64)
        if name: job["name"] = name
        jobs = self.owner.jobs.get(id, [])
        jobs.append(job)
        self.owner.jobs[id] = jobs
        appier.notify("jobs:%s" % id)

    @appier.route("/nodes/<str:id>/print", "OPTIONS")
    def print_default_o(self, id):
        return ""

    @appier.route("/nodes/<str:id>/print_hello", ("GET", "POST"), json = True)
    @appier.ensure(token = "admin")
    def print_hello_default(self, id):
        self.set_field("data_b64", HELLO_WORLD_B64)
        self.set_field("name", "hello_world")
        self.print_default(id)

    @appier.route("/nodes/<str:id>/printers/print", ("GET", "POST"), json = True)
    @appier.ensure(token = "admin")
    def print_printer_f(self, id):
        printer = self.field("printer")
        return self.print_printer(id, printer)

    @appier.route("/nodes/<str:id>/printers/print", "OPTIONS")
    def print_printer_of(self, id):
        printer = self.field("printer")
        return self.print_printer_o(id, printer)

    @appier.route("/nodes/<str:id>/printers/print_hello", ("GET", "POST"), json = True)
    @appier.ensure(token = "admin")
    def print_hello_printer_f(self, id):
        printer = self.field("printer")
        return self.print_hello_printer(id, printer)

    @appier.route("/nodes/<str:id>/printers/<str:printer>/print", ("GET", "POST"), json = True)
    @appier.ensure(token = "admin")
    def print_printer(self, id, printer):
        data_b64 = self.field("data_b64", mandatory = True, not_empty = True)
        name = self.field("name", None)
        name = name or str(uuid.uuid4())
        job = dict(
            data_b64 = data_b64,
            printer = printer
        )
        if name: job["name"] = name
        jobs = self.owner.jobs.get(id, [])
        jobs.append(job)
        self.owner.jobs[id] = jobs
        appier.notify("jobs:%s" % id)

    @appier.route("/nodes/<str:id>/printers/<str:printer>/print", "OPTIONS")
    def print_printer_o(self, id, printer):
        return ""

    @appier.route("/nodes/<str:id>/printers/<str:printer>/print_hello", ("GET", "POST"), json = True)
    @appier.ensure(token = "admin")
    def print_hello_printer(self, id, printer):
        self.set_field("data_b64", HELLO_WORLD_B64)
        self.set_field("name", "hello_world")
        self.print_printer(id, printer)

    @appier.coroutine
    def wait_jobs(self, id):
        while True:
            jobs = self.owner.jobs.pop(id, [])
            if jobs: break
            for value in appier.wait("jobs:%s" % id): yield value
        yield json.dumps(jobs)



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import exceptions
from . import system
from . import visitor

from .exceptions import PrintingBinieException, InvalidContextInformationName
from .system import PrintingBinie
from .visitor import Visitor



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import ast

from ..common.base import *

class Visitor(object):
    """
    The visitor class.
    """

    node_method_map = {}
    """ The node method map """

    visit_childs = True
    """ The visit childs flag """

    visit_next = True
    """ The visit next flag """

    visit_index = 0
    """ The visit index, for multiple visits """

    def __init__(self):
        self.node_method_map = {}
        self.visit_childs = True
        self.visit_next = True
        self.visit_index = 0

        self.update_node_method_map()

    def update_node_method_map(self):
        # retrieves the class of the current instance
        self_class = self.__class__

        # retrieves the names of the elements for the current class
        self_class_elements = dir(self_class)

        # iterates over all the name of the elements
        for self_class_element in self_class_elements:
            # retrieves the real element value
            self_class_real_element = getattr(self_class, self_class_element)

            # in case the current class real element does not contain
            # an AST node class reference must continue the loop
            if not hasattr(self_class_real_element, "ast_node_class"): continue

            # retrieves the AST node class from the current class real element
            # and sets it in the node method map
            ast_node_class = getattr(self_class_real_element, "ast_node_class")
            self.node_method_map[ast_node_class] = self_class_real_element

    @dispatch_visit()
    def visit(self, node):
        print("unrecognized element node of type " + node.__class__.__name__)

    def before_visit(self, node):
        self.visit_childs = True
        self.visit_next = True

    def after_visit(self, node):
        pass

    @visited(ast.AstNode)
    def visit_ast_node(self, node):
        print("AstNode: " + str(node))

    @visited(ast.GenericElement)
    def visit_generic_element(self, node):
        print("GenericElement: " + str(node))

    @visited(ast.PrintingDocument)
    def visit_printing_document(self, node):
        print("PrintingDocument: " + str(node))

    @visited(ast.Block)
    def visit_block(self, node):
        print("Block: " + str(node))

    @visited(ast.Paragraph)
    def visit_paragraph(self, node):
        print("Paragraph: " + str(node))

    @visited(ast.Line)
    def visit_line(self, node):
        print("Line: " + str(node))

    @visited(ast.Text)
    def visit_text(self, node):
        print("Text: " + str(node))

    @visited(ast.Image)
    def visit_image(self, node):
        print("Image: " + str(node))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import ast
from . import exceptions
from . import parser
from . import system
from . import visitor

from .ast import AstNode, GenericElement, PrintingDocument, Block, Paragraph, Line,\
    Text, Image
from .exceptions import PrintingManagerException, PrintingPluginNotAvailable
from .parser import Parser, valid_node
from .system import PrintingManager
from .visitor import Visitor



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import visitor

PRINTING_NAME = "binie"
""" The printing name """

class PrintingBinie(object):
    """
    The printing binie class, responsible for the handling
    of the front-end of the printing to binie support.
    """

    def get_name(self):
        return PRINTING_NAME

    def print_test(self, options = {}):
        pass

    def print_test_image(self, image_path, options = {}):
        pass

    def print_language(self, printing_document, options = {}):
        # creates the PDF printing visitor then sets the
        # provided printing options in the visitor
        _visitor = visitor.Visitor()
        _visitor.set_options(options)

        # accepts the visitor in the printing document,
        # using double visiting mode
        printing_document.accept_double(_visitor)



#!/usr/bin/python
# -*- coding: utf-8 -*-

class PrintingPdfException(Exception):
    """
    The printing PDF exception class.
    """

    message = None
    """ The exception's message """

class InvalidContextInformationName(PrintingPdfException):
    """
    The invalid context information name class.
    """

    def __init__(self, message):
        """
        Constructor of the class.

        :type message: String
        :param message: The message to be printed.
        """

        PrintingPdfException.__init__(self)
        self.message = message

    def __str__(self):
        """
        Returns the string representation of the class.

        :rtype: String
        :return: The string representation of the class.
        """

        return "Invalid context information name - %s" % self.message

class InvalidFont(PrintingPdfException):
    """
    The invalid font class.
    """

    def __init__(self, message):
        """
        Constructor of the class.

        :type message: String
        :param message: The message to be printed.
        """

        PrintingPdfException.__init__(self)
        self.message = message

    def __str__(self):
        """
        Returns the string representation of the class.

        :rtype: String
        :return: The string representation of the class.
        """

        return "Invalid font - %s" % self.message



#!/usr/bin/python
# -*- coding: utf-8 -*-

import base64

import appier

MIME = dict(
    binie = "text/x-binie",
    pdf = "application/pdf"
)
""" Map defining the association between the print
format naming and the associated base mime type value
(note that this value may be complemented with base64) """

EXAMPLE = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
    <printing_document name=\"hello_world\" font=\"Calibri\" font_size=\"9\">\
        <paragraph text_align=\"center\">\
            <line><text>Hello World</text></line>\
        </paragraph>\
    </printing_document>"
""" Example XML string that should display an hello world
message using the XML printing language """

class DocumentController(appier.Controller):

    def __init__(self, owner, *args, **kwargs):
        appier.Controller.__init__(self, owner, *args, **kwargs)
        self.manager = None

    @appier.route("/documents/example.<format>", "GET")
    def example(self, format):
        return self.send_print(EXAMPLE, format)

    @appier.route("/documents.<format>", "POST")
    def convert(self, format):
        # retrieves the current request reference and then
        # uses it to retrieve its "raw" data, that should
        # contain the XML string for the generation of the
        # of binie result and then sends the value for print
        request = self.get_request()
        data = request.get_data()
        return self.send_print(data, format = format)

    def send_print(self, data, format = "binie"):
        # retrieves the various optional fields for printing
        # and then parses them creating the composite values
        # (should include the size tuple)
        b64 = self.field("base64", False, cast = bool)
        width = self.field("width", 0.0, cast = float)
        height = self.field("height", 0.0, cast = float)
        has_size = width > 0.0 and width > 0.0

        mime = self.get_mime(format, b64 = base64)
        manager = self.get_manager()

        data = data
        file = appier.legacy.BytesIO()
        options = dict(name = format, file = file)
        if has_size: options["size"] = (width, height)

        manager.print_language(data, options)
        value = file.getvalue()
        value = base64.b64encode(value) if b64 else value

        self.content_type(mime)
        return value

    def get_mime(self, format, b64 = False):
        mime = MIME.get(format, "application/octet-stream")
        mime = mime + "-base64" if b64 else mime
        return mime

    def get_manager(self):
        import colony_print
        if self.manager: return self.manager
        self.manager = colony_print.PrintingManager()
        self.manager.load()
        return self.manager



#!/usr/bin/python
# -*- coding: utf-8 -*-

from . import exceptions
from . import system
from . import visitor

from .exceptions import PrintingPdfException, InvalidContextInformationName, InvalidFont
from .system import PrintingPDF
from .visitor import Visitor



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import setuptools

setuptools.setup(
    name = "colony-print",
    version = "0.2.0",
    author = "Hive Solutions Lda.",
    author_email = "development@hive.pt",
    description = "Colony Print Infra-structure",
    license = "Apache License, Version 2.0",
    keywords = "colony print native",
    url = "http://colony-print.hive.pt",
    zip_safe = False,
    packages = [
        "colony_print",
        "colony_print.controllers",
        "colony_print.printing",
        "colony_print.printing.binie",
        "colony_print.printing.common",
        "colony_print.printing.manager",
        "colony_print.printing.pdf"
    ],
    test_suite = "colony_print.test",
    package_dir = {
        "" : os.path.normpath("src")
    },
    package_data = {
        "colony_print" : [
            "static/example/*",
            "static/example/js/*",
            "static/example/xml/*"
        ]
    },
    install_requires = [
        "appier",
        "appier-extras",
        "jinja2",
        "pillow",
        "reportlab"
    ],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7"
    ],
    long_description = open(os.path.join(os.path.dirname(__file__), "README.md"), "rb").read().decode("utf-8"),
    long_description_content_type = "text/markdown"
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier

class PrinterController(appier.Controller):

    @appier.route("/printers", "GET", json = True)
    @appier.ensure(token = "admin")
    def list(self):
        return self.npcolony.get_devices()

    @appier.route("/printers/hello", "GET", json = True)
    @appier.ensure(token = "admin")
    def hello(self):
        self.npcolony.print_hello()

    @appier.route("/printers/print", "GET", json = True)
    @appier.ensure(token = "admin")
    def print_document_f(self):
        printer = self.field("printer")
        return self.print_document(printer)

    @appier.route("/printers/<str:printer>/print", ("GET", "POST"), json = True)
    @appier.ensure(token = "admin")
    def print_document(self, printer):
        data_b64 = self.field("data_b64")
        self.npcolony.print_printer_base64(printer, data_b64)

    @property
    def npcolony(self):
        import npcolony
        return npcolony



#!/usr/bin/python
# -*- coding: utf-8 -*-

import time
import logging

import appier

BASE_URL = "https://print.bemisc.com/"

SLEEP_TIME = 3.0

class ColonyPrintNode(object):

    def __init__(self, sleep_time = SLEEP_TIME):
        self.sleep_time = sleep_time

    def loop(self):
        logging.basicConfig(
            format = "%(asctime)s [%(levelname)s] %(message)s",
            level = logging.DEBUG
        )

        base_url = appier.conf("BASE_URL", BASE_URL)
        secret_key = appier.conf("SECRET_KEY", None)
        node_id = appier.conf("NODE_ID", "node")
        node_name = appier.conf("NODE_NAME", "node")
        node_location = appier.conf("NODE_LOCATION", "undefined")

        headers = dict()
        if secret_key: headers["X-Secret-Key"] = secret_key

        while True:
            try:
                logging.info("Submitting node information")
                appier.post(
                    base_url + "nodes/%s" % node_id,
                    data_j = dict(
                        name = node_name,
                        location = node_location
                    ),
                    headers = headers
                )
                logging.info("Retrieving jobs for node '%s'" % node_id)
                jobs = appier.get(
                    base_url + "nodes/%s/jobs" % node_id,
                    headers = headers,
                    timeout = 600
                )
                logging.info("Retrieved %d jobs for node '%s'" % (len(jobs), node_id))
                for job in jobs: self.print_job(job)
            except Exception as exception:
                logging.info("Exception while looping '%s'" % str(exception))
                logging.info("Sleeping for %.2f seconds" % self.sleep_time)
                time.sleep(self.sleep_time)

    def print_job(self, job):
        data_b64 = job["data_b64"]
        name = job.get("name", "undefined")
        printer = job.get("printer", None)
        printer_s = printer if printer else "default"
        logging.info("Printing job '%s' with '%s' printer" % (name, printer_s))
        if printer: self.npcolony.print_printer_base64(printer, data_b64)
        else: self.npcolony.print_base64(data_b64)

    @property
    def npcolony(self):
        import npcolony
        return npcolony

if __name__ == "__main__":
    node = ColonyPrintNode()
    node.loop()
else:
    __path__ = []



#!/usr/bin/python
# -*- coding: utf-8 -*-

import appier
import appier_extras

class ColonyPrintApp(appier.APIApp):

    def __init__(self, *args, **kwargs):
        appier.APIApp.__init__(
            self,
            name = "colony-print",
            parts = (
                appier_extras.AdminPart,
            ),
            *args, **kwargs
        )
        self.nodes = dict()
        self.jobs = dict()

if __name__ == "__main__":
    app = ColonyPrintApp()
    app.serve()
else:
    __path__ = []
